<?php

namespace Drupal\migrate_source_ical\Plugin\migrate;

use Drupal\migrate\MigrateException;
use GuzzleHttp\Exception\RequestException;
use ICal\ICal;

/**
 * Object to retrieve and iterate over JSON data.
 */
class IcalReader {

  /**
   * The HTTP Client.
   *
   * @var JSONClientInterface resource
   */
  protected $client;

  /**
   * The request headers.
   *
   * @var array
   */
  protected $headers = [];

  /**
   * Source configuration.
   *
   * @var array
   */
  protected $configuration;

  /**
   * The field name that is a unique identifier.
   *
   * @var string
   */
  protected $identifier = '';

  /**
   * The depth of the identifier in the source data.
   *
   * @var string
   */
  protected $identifierDepth = '';

  /**
   * Set the configuration created by the JSON source.
   *
   * @param array $configuration
   *   The source configuration.
   *
   * @throws \Drupal\migrate\MigrateException
   */
  public function __construct(array $configuration) {

    // Pull out the values this reader will care about.
    if (!isset($configuration['identifier'])) {
      throw new MigrateException('The source configuration must include the identifier.');
    }
    if (!isset($configuration['identifierDepth'])) {
      throw new MigrateException('The source configuration must include the identifierDepth.');
    }
    $this->identifier = $configuration['identifier'];
    $this->identifierDepth = $configuration['identifierDepth'];

    // Store the rest of the configuration.
    $this->configuration = $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function getClient() {
    return $this->client;
  }

  /**
   * Return the identifier for this JSON source.
   */
  public function getIdentifier() {
    return $this->identifier ?? '';
  }

  /**
   * Return the identifier depth for this JSON source.
   */
  public function getIdentifierDepth() {
    return $this->identifierDepth ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFields($url) {

    $iterator = $this->getSourceData($url);

    // Recurse through the result array. When there is an array of items at the
    // expected depth that has the expected identifier as one of the keys,
    // pull that array out as a distinct item.
    // $identifier = $this->getIdentifier();
    // $identifierDepth = $this->getIdentifierDepth();
    $items = [];
    while ($iterator->valid()) {
      $iterator->next();
      $item = $iterator->current();
      if (is_array($item)) {
        $items[] = $item;
      }
    }

    return $iterator;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldsIterator($url) {
    return $this->getSourceFields($url);
  }

  /**
   * Get the source data for reading.
   *
   * @param string $url
   *   The URL to read the source data from.
   *
   * @return \RecursiveIteratorIterator|resource
   *   Returns a resource.
   *
   * @throws \Drupal\migrate\MigrateException
   */
  public function getSourceData($url) {
    try {
      $response = [];
      $ical = new ICal($url);
      foreach ($ical->events() as $events) {
        $response[] = (array) $events;
      }
      // Each object returns and event array.
      $obj = new \ArrayIterator($response);
      return $obj;
    }
    catch (RequestException $e) {
      throw new MigrateException($e->getMessage(), $e->getCode(), $e);
    }
  }

}
