<?php

namespace Drupal\migrate_source_ical\Plugin\migrate\process;

use Drupal\migrate\Plugin\migrate\process\FormatDate;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\Row;

/**
 * Convert iCal-formatted dates into another format.
 *
 * A copy of the core format_date plugin which automatically detects the input
 * format.
 *
 * Available configuration keys:
 * - to_format: The destination format.
 *   @link http://php.net/manual/datetime.createfromformat.php \DateTime::createFromFormat. @endlink
 * - timezone: (deprecated) String identifying the required time zone, see
 *   DateTimePlus::__construct(). The timezone configuration key is deprecated
 *   in Drupal 8.4.x and will be removed before Drupal 9.0.0, use from_timezone
 *   and to_timezone instead.
 * - from_timezone: String identifying the required source time zone, see
 *   DateTimePlus::__construct().
 * - to_timezone: String identifying the required destination time zone, see
 *   DateTimePlus::__construct().
 * - settings: keyed array of settings, see DateTimePlus::__construct().
 *
 * Configuration keys from_timezone and to_timezone are both optional. Possible
 * input variants:
 * - Both from_timezone and to_timezone are empty. Date will not be converted
 *   and be treated as date in default timezone.
 * - Only from_timezone is set. Date will be converted from timezone specified
 *   in from_timezone key to the default timezone.
 * - Only to_timezone is set. Date will be converted from the default timezone
 *   to the timezone specified in to_timezone key.
 * - Both from_timezone and to_timezone are set. Date will be converted from
 *   timezone specified in from_timezone key to the timezone specified in
 *   to_timezone key.
 *
 * Example usage:
 *
 * @code
 * process:
 *   field_date:
 *     plugin: ical_format_date
 *     to_format: Y-m-d\TH:i:s
 *     source: event_date
 * @endcode
 *
 * @see \DateTime::createFromFormat()
 * @see \Drupal\Component\Datetime\DateTimePlus::__construct()
 * @see \Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface
 * @see \Drupal\migrate\Plugin\MigrateProcessInterface
 * @see \Drupal\migrate\Plugin\migrate\process\FormatDate
 *
 * @MigrateProcessPlugin(
 *   id = "ical_format_date"
 * )
 */
class IcalFormatDate extends FormatDate {

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    if (empty($value) && $value !== '0' && $value !== 0) {
      return '';
    }

    // Set the 'from_format' automatically based upon the input string.
    $this->configuration['from_format'] = $this->determineFromFormat($value);

    // Let the format_date plugin do most of the work.
    return parent::transform($value, $migrate_executable, $row, $destination_property);
  }

  /**
   * Determine the date format to use for the input string.
   *
   * @param string $value
   *   The source string that is being imported.
   *
   * @return string
   *   The date format to use, using standard PHP notation; defaults to 'Ymd'.
   */
  private function determineFromFormat($value) {
    $length = strlen($value);

    // Date strings which include the full date, time and "Z".
    if ($length == 16) {
      $from_format = 'Ymd\THis\Z';
    }
    elseif ($length == 15) {
      $from_format = 'Ymd\THis';
    }
    else {
      $from_format = 'Ymd';
    }
    return $from_format;
  }

}
