<?php

namespace Drupal\Tests\migrate_source_ical\Kernel\Plugin\migrate\source;

use Drupal\datetime\Plugin\Field\FieldType\DateTimeItem;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\KernelTests\KernelTestBase;
use Drupal\migrate\MigrateExecutable;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\node\Entity\NodeType;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;

/**
 * @coversDefaultClass \Drupal\migrate_source_ical\Plugin\migrate\source\Ical
 *
 * @group migrate_source_ical
 */
class IcalKernelTest extends KernelTestBase {

  use ContentTypeCreationTrait;

  /**
   * Node type i.e event.
   *
   * @var \Drupal\node\Entity\NodeType
   */
  protected NodeType $bundle;

  /**
   * Node entity type.
   *
   * @var string
   */
  protected string $entityTypeId;

  /**
   * A field storage to use in this test class.
   *
   * @var \Drupal\field\Entity\FieldStorageConfig
   */
  protected $fieldStorage;

  /**
   * The field used in this test class.
   *
   * @var \Drupal\field\Entity\FieldConfig
   */
  protected $field;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'migrate',
    'migrate_source_ical',
    'node',
    'user',
    'system',
    'field',
    'text',
    'datetime',
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();
    $this->installSchema('node', []);
    $this->installSchema('user', []);
    $this->installSchema('system', []);
    $this->installSchema('field', []);
    $this->installSchema('user', []);
    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installConfig(['node', 'user']);

    $this->entityTypeId = 'node';
    $this->bundle = $this->createContentType(['type' => 'event']);

    // Create a field with settings to validate.
    $this->fieldStorage = FieldStorageConfig::create([
      'field_name' => 'field_event_date',
      'type' => 'datetime',
      'entity_type' => $this->entityTypeId,
      'settings' => ['datetime_type' => DateTimeItem::DATETIME_TYPE_DATE],
    ]);
    $this->fieldStorage->save();
    $this->field = FieldConfig::create([
      'field_storage' => $this->fieldStorage,
      'bundle' => $this->bundle->id(),
      'settings' => [
        'default_value' => 'blank',
      ],
    ]);
    $this->field->save();
  }

  /**
   * Tests if source plugin exists.
   */
  public function testSourcePluginExists() {
    /** @var \Drupal\migrate\Plugin\MigratePluginManagerInterface $migrationSourceManager */
    $migrationSourceManager = $this->container->get('plugin.manager.migrate.source');
    $this->assertTrue($migrationSourceManager->hasDefinition('ical'));
  }

  /**
   * Tests if source data is converted into entities.
   */
  public function testSourcePlugin() {
    $calendar = $this->getModulePath('migrate_source_ical') . '/tests/calendar.ics';
    $definition = [
      'source' => [
        'plugin' => 'ical',
        'path' => $calendar,
        'identifier' => 'upc',
        'identifierDepth' => 1,
        'ids' => [
          'uid',
        ],
        'fields' => [
          'uid' => 'UID',
          'dtstart' => 'Date Start',
          'summary' => 'Summary',
        ],
      ],
      'process' => [
        'title' => 'summary',
        'field_event_date' => [
          'value' => [
            'plugin' => 'ical_format_date',
            'to_format' => 'Y-m-d',
            'source' => 'dtstart',
          ],
        ],
      ],
      'destination' => [
        'plugin' => 'entity:node',
        'default_bundle' => 'event',
      ],
    ];

    $migration = \Drupal::service('plugin.manager.migration')->createStubMigration($definition);

    $executable = new MigrateExecutable($migration);
    $result = $executable->import();
    // Check that the migration has completed.
    $this->assertEquals(MigrationInterface::RESULT_COMPLETED, $result);

    /** @var \Drupal\migrate\Plugin\MigrateIdMapInterface $id_map_plugin */
    $id_map_plugin = $migration->getIdMap();
    // Check if date is properly stored in the event field.
    $map_row = $id_map_plugin->getRowBySource(['uid' => 'ical_event01']);
    $this->assertNotNull($map_row['destid1']);
    $node1 = $this->container->get('entity_type.manager')->getStorage('node')
      ->load($map_row['destid1']);
    $this->assertEquals('2024-12-09', $node1->field_event_date->value);

    $map_row = $id_map_plugin->getRowBySource(['uid' => 'ical_event02']);
    $this->assertNotNull($map_row['destid1']);
    $node2 = $this->container->get('entity_type.manager')->getStorage('node')
      ->load($map_row['destid1']);
    $this->assertEquals('2025-03-24', $node2->field_event_date->value);
  }

}
